#include <windows.h>

// This one below seems to work only with Visual C++ 2003 ( and not VC++ Express ) 
//#define STRSAFE_NO_DEPRECATE
// Use this one on VC++ Express
#define _CRT_NONSTDC_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE
#pragma warning ( disable : 4996 ) // 'function' Deprecated 

#include <stdio.h>
#include <tchar.h>

// #define  EMA_STATIC_LINK 
#include "EMADevice.h"

EMADevice * Z800 = NULL ;

void InitializeHeadset ( ) 
{ 
	Z800 = new EMADevice ( ) ;
	if ( NULL == Z800 ) 
	{ 
		HRESULT hr = EMADevice::GetLastError (  ) ; 
        MessageBox ( NULL , "Could not connect to headset" , "eMagin SDK - Yaw Pitch Roll Sample" , MB_OK ) ; 
		exit ( hr ) ; 
	} 
} 

void DeInitializeHeadset ( ) 
{ 
    // destroy device 
    if ( NULL != Z800 ) 
    { 
        delete Z800 ; 
    } 
} 

void GetHeadsetData ( double * Pitch, double * Yaw, double * Roll,
					 double * AccX, double * AccY, double * AccZ,
					 double * AngX, double * AngY, double * AngZ, double * North ) 					 

{
	EMADeviceData DeviceData ; 
	Z800->PollHeadTrackerRawData ( & DeviceData ) ; 

	// Convert the raw sensor data to the DirectX coordinate system
	DeviceData.ToDirectX ( ) ;

	// Return the angles 
    * Yaw   = ( double ) DeviceData.Yaw ;
    * Pitch = ( double ) DeviceData.Pitch ;
    * Roll  = ( double ) DeviceData.Roll ;

	* AccX  = ( double ) DeviceData.AccX ;
    * AccY  = ( double ) DeviceData.AccY ;
    * AccZ  = ( double ) DeviceData.AccZ ;
	* AngX  = ( double ) DeviceData.AngX ;
    * AngY  = ( double ) DeviceData.AngY ;
    * AngZ  = ( double ) DeviceData.AngZ ;

    * North  = ( double ) DeviceData.North ;
}


LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

// WinMain(), our entry point 
//
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE /* hPrevInstance */ , LPSTR /* szCmdLine */ , int iCmdShow) 
{
    static char szAppName[] = "Headset";
    HWND        hwnd;
    MSG         msg;
    WNDCLASSEX  wndclass;

    // Fill in WNDCLASSEX struct members
    wndclass.cbSize         = sizeof(wndclass);
    wndclass.style          = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc    = WndProc;
    wndclass.cbClsExtra     = 0;
    wndclass.cbWndExtra     = 0;
    wndclass.hInstance      = hInstance;
    wndclass.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
    wndclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);
    wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wndclass.hbrBackground  = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wndclass.lpszClassName  = szAppName;
    wndclass.lpszMenuName   = NULL ;  

    // Register a new window class with Windows  
    RegisterClassEx(&wndclass);
    // Create a window based on our new class  
    hwnd = CreateWindow ( szAppName, "Yaw Pitch Roll", 
            WS_OVERLAPPEDWINDOW, 
            CW_USEDEFAULT, CW_USEDEFAULT,
            210, 480,
            NULL, NULL, hInstance, NULL);
    // Show and update our window
    ShowWindow(hwnd, iCmdShow);
    UpdateWindow(hwnd);

    InitializeHeadset ( ) ; 

    // Run a timer to refresh headset values on screen 
    SetTimer(hwnd, 0, 200, NULL);

    // Retrieve and process messages until we get WM_QUIT
    while ( GetMessage(&msg, NULL, 0, 0) ) 
    {
        // dont bother with accels 
        TranslateMessage(&msg);    // for certain keyboard messages  
        DispatchMessage(&msg);     // send message to WndProc        
    } 
    KillTimer( hwnd , 0) ;
    // Cleanup COM and release the interface
    DeInitializeHeadset ( ) ; 
    //  Exit with status specified in WM_QUIT message
    return ( int ) msg.wParam;
}

void DrawWindow ( HDC hdc ) 
{
    if ( NULL != Z800 )
    {
        // Get headset data 
        double Pitch, Yaw, Roll ; 
        double AccX, AccY, AccZ, AngX, AngY, AngZ, North ;

		static double tPitch=0, tYaw=0, tRoll=0 ; 
        static double tAccX=0, tAccY=0, tAccZ=0, tAngX=0, tAngY=0, tAngZ=0, tNorth=0 ;

		
		GetHeadsetData ( & Pitch, & Yaw, & Roll, & AccX, & AccY, & AccZ, & AngX, & AngY, & AngZ, & North ) ; 
        // Windows stuff: display values on screen 
        const TCHAR spaces [] = "                                     " ; 
        TextOut ( hdc, 20, 20, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20, 40, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20, 60, spaces, sizeof(spaces)-1 );

        TextOut ( hdc, 20,100, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,120, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,140, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,180, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,200, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,220, spaces, sizeof(spaces)-1 );
        TextOut ( hdc, 20,260, spaces, sizeof(spaces)-1 );

		TCHAR szMsg [MAX_PATH] ;
		int length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "Yaw : %+5.3f: %+5.3f" ) ,Yaw, tYaw-Yaw ) ; 
        TextOut ( hdc, 20, 20, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "Pitch: %+5.3f: %+5.3f" ) , Pitch, tPitch-Pitch ) ; 
        TextOut ( hdc, 20, 40, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "Roll : %+5.3f: %+5.3f" ) , Roll, tRoll-Roll ) ; 
        TextOut ( hdc, 20, 60, szMsg , length );

        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AccX: %+5.3f: %+5.3f" ) , AccX, tAccX-AccX ) ; 
        TextOut ( hdc, 20,100, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AccY: %+5.3f: %+5.3f" ) , AccY, tAccY-AccY ) ; 
        TextOut ( hdc, 20,120, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AccZ: %+5.3f: %+5.3f" ) , AccZ, tAccZ-AccZ ) ; 
        TextOut ( hdc, 20,140, szMsg , length );

		length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AngX: %+5.1f: %+5.1f" ) , AngX, tAngX-AngX ) ; 
        TextOut ( hdc, 20,180, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AngY: %+5.1f: %+5.1f" ) , AngY, tAngY-AngY ) ; 
        TextOut ( hdc, 20,200, szMsg , length );
        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "AngZ: %+5.1f: %+5.1f" ) , AngZ, tAngZ-AngZ ) ; 
        TextOut ( hdc, 20,220, szMsg , length );

        length = _sntprintf ( szMsg , MAX_PATH , TEXT ( "North: %+5.3f: %+5.3f" ) , North, tNorth-North ) ; 
        TextOut ( hdc, 20,260, szMsg , length );

		tPitch=Pitch; tYaw=Yaw; tRoll=Roll; 
        tAccX=AccX; tAccY=AccY; tAccZ=AccZ;
		tAngX=AngX; tAngY=AngY; tAngZ=AngZ; tNorth=North;

	}
    else
    {
        const TCHAR spaces [] = "                             " ; 
        TextOut ( hdc, 40, 40, spaces, sizeof(spaces)-1 );
        const TCHAR Msg [ ] = "Headset not ready yet." ; 
        TextOut ( hdc, 40 , 40, Msg , sizeof ( Msg ) - 1 );
    }
} 

// Window procedure
//
LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam) 
{
    // Switch according to what type of message we have received
    switch ( iMsg ) 
    {
      case WM_TIMER:      // Invalidate the window 
          DrawWindow ( GetDC ( hwnd ) ) ; 
          return 0 ; 

      case WM_PAINT:      // We receive WM_PAINT every time window is updated
        {
            PAINTSTRUCT ps;
            HDC hdc = BeginPaint(hwnd, &ps);
            DrawWindow ( hdc ) ; 
            EndPaint ( hwnd, & ps );
            return 0;
        }
      case WM_DESTROY: // Window has been destroyed, so exit cleanly
            PostQuitMessage(0);
            return 0;
      }
      // Send any messages we don't handle to default window procedure
      return DefWindowProc(hwnd, iMsg, wParam, lParam);
}
